from sqlalchemy import Column, String, Float, Boolean, ForeignKey, Integer, DateTime, Enum, text, Table
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base
from api.v1.models.base_model import BaseTableModel
from api.db.database import Base
from datetime import datetime
import enum as PyEnum

from api.v1.models.user import roles
#from api.v1.models.task_association import task_user_association

# Task status ENUM
class TaskStatus(PyEnum.Enum):
    PENDING = "Pending"
    IN_PROGRESS = "In Progress"
    COMPLETED = "Completed"

# Task priority ENUM
class TaskPriority(PyEnum.Enum):
    LOW = "Low"
    MEDIUM = "Medium"
    HIGH = "High"

statuses = Enum(TaskStatus, name="task_status", create_type=True)
priorities = Enum(TaskPriority, name="task_priority", create_type=True)

class Task(BaseTableModel):
    __tablename__ = "tasks"

    title = Column(String, nullable=False)
    description = Column(String, nullable=True)
    image_url = Column(String, nullable=True)
    status = Column(statuses, default="Pending", nullable=False)
    priority = Column(priorities, default="Medium", nullable=False)
    due_date = Column(DateTime, nullable=True)
    required_user_role = Column(roles, nullable=True)
    budget = Column(Float, nullable=False, server_default=text("0.0"))
    currency = Column(String, nullable=False, server_default="usdt")
    participants = Column(Integer, nullable=True,server_default=text("0"))

    user_id = Column(ForeignKey('users.id'), nullable=True)

    is_deleted = Column(Boolean, server_default=text("false"))
    created_at = Column(DateTime, default=datetime.utcnow, nullable=False)
    updated_at = Column(DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)

    assigned_users = relationship("TaskUserAssociation", back_populates="task")

    def to_dict(self):
        obj_dict = super().to_dict()
        obj_dict['assigned_users'] = [user.to_dict() for user in self.assigned_users]
        return obj_dict
